@@ -84,6 +84,7 @@ main(int argc, char **argv)
 	char **p;
 	pfarg_pmc_t pc[NUM_PMCS];
 	pfarg_pmd_t pd[NUM_PMDS];
+	uint64_t pdo[NUM_PMDS];
 	pfarg_ctx_t ctx[1];
 	pfarg_load_t load_args;
 	pfmlib_input_param_t inp;
@@ -91,7 +92,7 @@ main(int argc, char **argv)
 	pfmlib_options_t pfmlib_options;
 	unsigned int which_cpu;
 	int ret, ctx_fd;
-	unsigned int i, j;
+	unsigned int i, l;
 	unsigned int num_counters;
 	char name[MAX_EVT_NAME_LEN];
 
@@ -103,7 +104,7 @@ main(int argc, char **argv)
 		fatal_error("Cannot initialize library: %s\n", pfm_strerror(ret));
 
 	pfm_get_num_counters(&num_counters);
-	
+
 	/*
 	 * pass options to library (optional)
 	 */
@@ -114,6 +115,7 @@ main(int argc, char **argv)
 
 	memset(pc, 0, sizeof(pc));
 	memset(pd, 0, sizeof(pd));
+	memset(pdo, 0, sizeof(pdo));
 	memset(ctx, 0, sizeof(ctx));
 	memset(&inp,0, sizeof(inp));
 	memset(&outp,0, sizeof(outp));
@@ -125,15 +127,15 @@ main(int argc, char **argv)
 	if (argc > 1) {
 		p = argv+1;
 		for (i=0; *p ; i++, p++) {
-			if (pfm_find_event(*p, &inp.pfp_events[i].event) != PFMLIB_SUCCESS) {
+			if (pfm_find_full_event(*p, &inp.pfp_events[i]) != PFMLIB_SUCCESS) {
 				fatal_error("Cannot find %s event\n", *p);
 			}
 		}
 	} else {
-		if (pfm_get_cycle_event(&inp.pfp_events[0].event) != PFMLIB_SUCCESS) {
+		if (pfm_get_cycle_event(&inp.pfp_events[0]) != PFMLIB_SUCCESS) {
 			fatal_error("cannot find cycle event\n");
 		}
-		if (pfm_get_inst_retired_event(&inp.pfp_events[1].event) != PFMLIB_SUCCESS) {
+		if (pfm_get_inst_retired_event(&inp.pfp_events[1]) != PFMLIB_SUCCESS) {
 			fatal_error("cannot find inst retired event\n");
 		}
 		i = 2;
@@ -180,9 +182,8 @@ main(int argc, char **argv)
 	 * the call.
 	 */
 	ret = pin_cpu(getpid(), which_cpu);
-	if (ret == -1) {
+	if (ret == -1)
 		fatal_error("cannot set affinity to CPU%d: %s\n", which_cpu, strerror(errno));
-	}
 	/*
 	 * after the call the task is pinned to which_cpu
 	 */
@@ -217,104 +218,82 @@ main(int argc, char **argv)
 	/*
 	 * let the library figure out the values for the PMCS
 	 */
-	if ((ret=pfm_dispatch_events(&inp, NULL, &outp, NULL)) != PFMLIB_SUCCESS) {
+	if ((ret=pfm_dispatch_events(&inp, NULL, &outp, NULL)) != PFMLIB_SUCCESS)
 		fatal_error("cannot configure events: %s\n", pfm_strerror(ret));
-	}
 
 	/*
 	 * Now prepare the argument to initialize the PMDs and PMCS.
-	 * We must pfp_pmc_count to determine the number of PMC to intialize.
-	 * We must use pfp_event_count to determine the number of PMD to initialize.
-	 * Some events causes extra PMCs to be used, so  pfp_pmc_count may be >= pfp_event_count.
-	 *
-	 * This step is new compared to libpfm-2.x. It is necessary because the library no
-	 * longer knows about the kernel data structures.
+	 * We use pfp_pmc_count to determine the number of PMC to intialize.
+	 * We use pfp_pmd_count to determine the number of PMD to initialize.
+	 * Some events/features may cause extra PMCs to be used, leading to:
+	 * 	- pfp_pmc_count may be >= pfp_event_count
+	 * 	- pfp_pmd_count may be >= pfp_event_count
 	 */
 	for (i=0; i < outp.pfp_pmc_count; i++) {
 		pc[i].reg_num   = outp.pfp_pmcs[i].reg_num;
 		pc[i].reg_value = outp.pfp_pmcs[i].reg_value;
 	}
-	/*
-	 * figure out pmd mapping from output pmc
-	 */
-	for (i=0, j=0; i < inp.pfp_event_count; i++) {
-		pd[i].reg_num   = outp.pfp_pmcs[j].reg_pmd_num;
-		for(; j < outp.pfp_pmc_count; j++)  if (outp.pfp_pmcs[j].reg_evt_idx != i) break;
-	}
-
+	for (i=0; i < outp.pfp_pmd_count; i++)
+		pd[i].reg_num = outp.pfp_pmds[i].reg_num;
 	/*
 	 * Now program the registers
-	 *
-	 * We don't use the save variable to indicate the number of elements passed to
-	 * the kernel because, as we said earlier, pc may contain more elements than
-	 * the number of events we specified, i.e., contains more thann coutning monitors.
 	 */
-	if (pfm_write_pmcs(ctx_fd, pc, outp.pfp_pmc_count) == -1) {
+	if (pfm_write_pmcs(ctx_fd, pc, outp.pfp_pmc_count) == -1)
 		fatal_error("pfm_write_pmcs error errno %d\n",errno);
-	}
-	/*
-	 * To be read, each PMD must be either written or declared
-	 * as being part of a sample (reg_smpl_pmds)
-	 */
-	if (pfm_write_pmds(ctx_fd, pd, inp.pfp_event_count) == -1) {
+
+	if (pfm_write_pmds(ctx_fd, pd, outp.pfp_pmd_count) == -1)
 		fatal_error("pfm_write_pmds error errno %d\n",errno);
-	}
 
 	/*
-	 * for system wide session, we can only attached to ourself
+	 * in system-wide mode, this field must provide the CPU the caller wants
+	 * to monitor. The kernel checks and if calling from the wrong CPU, the
+	 * call fails. The affinity is not affected.
 	 */
-	load_args.load_pid = getpid();
-
-	if (pfm_load_context(ctx_fd, &load_args) == -1) {
+	load_args.load_pid = which_cpu;
+	if (pfm_load_context(ctx_fd, &load_args) == -1)
 		fatal_error("pfm_load_context error errno %d\n",errno);
-	}
 
-	/*
-	 * start monitoring. We must go to the kernel because psr.pp cannot be
-	 * changed at the user level.
-	 */
-	if (pfm_start(ctx_fd, NULL) == -1) {
-		fatal_error("pfm_start error errno %d\n",errno);
-	}
 	printf("<monitoring started on CPU%d>\n", which_cpu);
 
-	printf("<press a key to stop monitoring>\n");
-	getchar();
-	/*
-	 * stop monitoring. We must go to the kernel because psr.pp cannot be
-	 * changed at the user level.
-	 */
-	if (pfm_stop(ctx_fd) == -1) {
-		fatal_error("pfm_stop error errno %d\n",errno);
-	}
+	for(l=0; l < 10; l++) {
+		/*
+		 * start monitoring
+		 */
+		if (pfm_start(ctx_fd, NULL) == -1)
+			fatal_error("pfm_start error errno %d\n",errno);
 
-	printf("<monitoring stopped on CPU%d>\n\n", which_cpu);
-	/*
-	 * now read the results
-	 */
-	if (pfm_read_pmds(ctx_fd, pd, inp.pfp_event_count) == -1) {
-		fatal_error( "pfm_read_pmds error errno %d\n",errno);
-		return -1;
-	}
+		sleep(2);
 
-	/*
-	 * print the results
-	 *
-	 * It is important to realize, that the first event we specified may not
-	 * be in PMD4. Not all events can be measured by any monitor. That's why
-	 * we need to use the pc[] array to figure out where event i was allocated.
-	 *
-	 */
-	for (i=0; i < inp.pfp_event_count; i++) {
-		pfm_get_event_name(inp.pfp_events[i].event, name, MAX_EVT_NAME_LEN);
-		printf("CPU%-2d PMD%u %20"PRIu64" %s\n",
-			which_cpu,
-			pd[i].reg_num,
-			pd[i].reg_value,
-			name);
+		/*
+		 * stop monitoring. 
+		 * changed at the user level.
+		 */
+		if (pfm_stop(ctx_fd) == -1)
+			fatal_error("pfm_stop error errno %d\n",errno);
+
+		/*
+		 * read the results
+		 */
+		if (pfm_read_pmds(ctx_fd, pd, inp.pfp_event_count) == -1)
+			fatal_error( "pfm_read_pmds error errno %d\n",errno);
+
+		/*
+		 * print the results
+		 */
+		puts("------------------------");
+		for (i=0; i < inp.pfp_event_count; i++) {
+			pfm_get_full_event_name(&inp.pfp_events[i], name, MAX_EVT_NAME_LEN);
+			printf("CPU%-2d PMD%-3u raw=%-20"PRIu64" delta=%-20"PRIu64" %s\n",
+					which_cpu,
+					pd[i].reg_num,
+					pd[i].reg_value,
+					pd[i].reg_value - pdo[i],
+					name);
+			pdo[i] = pd[i].reg_value;
+		}
 	}
 	/*
-	 * let's stop this now
+	 * destroy everything
 	 */
 	close(ctx_fd);
 
