@@ -1253,26 +1253,6 @@ pentium4_event_t pentium4_events[] = {
 			 "single-precisions operands.",
 		 .bit = 15,
 		},
-		{.name = "TAG0",
-		 .desc = "Tag this event with tag bit 0 "
-			 "for retirement counting with execution_event.",
-		 .bit = 16,
-		},
-		{.name = "TAG1",
-		 .desc = "Tag this event with tag bit 1 "
-			 "for retirement counting with execution_event.",
-		 .bit = 17,
-		},
-		{.name = "TAG2",
-		 .desc = "Tag this event with tag bit 2 "
-			 "for retirement counting with execution_event.",
-		 .bit = 18,
-		},
-		{.name = "TAG3",
-		 .desc = "Tag this event with tag bit 3 "
-			 "for retirement counting with execution_event.",
-		 .bit = 19,
-		},
 	 },
 	},
 
@@ -1288,26 +1268,6 @@ pentium4_event_t pentium4_events[] = {
 			 "double-precisions operands.",
 		 .bit = 15,
 		},
-		{.name = "TAG0",
-		 .desc = "Tag this event with tag bit 0 "
-			 "for retirement counting with execution_event.",
-		 .bit = 16,
-		},
-		{.name = "TAG1",
-		 .desc = "Tag this event with tag bit 1 "
-			 "for retirement counting with execution_event.",
-		 .bit = 17,
-		},
-		{.name = "TAG2",
-		 .desc = "Tag this event with tag bit 2 "
-			 "for retirement counting with execution_event.",
-		 .bit = 18,
-		},
-		{.name = "TAG3",
-		 .desc = "Tag this event with tag bit 3 "
-			 "for retirement counting with execution_event.",
-		 .bit = 19,
-		},
 	 },
 	},
 
@@ -1323,26 +1283,6 @@ pentium4_event_t pentium4_events[] = {
 			 "single-precisions operands.",
 		 .bit = 15,
 		},
-		{.name = "TAG0",
-		 .desc = "Tag this event with tag bit 0 "
-			 "for retirement counting with execution_event.",
-		 .bit = 16,
-		},
-		{.name = "TAG1",
-		 .desc = "Tag this event with tag bit 1 "
-			 "for retirement counting with execution_event.",
-		 .bit = 17,
-		},
-		{.name = "TAG2",
-		 .desc = "Tag this event with tag bit 2 "
-			 "for retirement counting with execution_event.",
-		 .bit = 18,
-		},
-		{.name = "TAG3",
-		 .desc = "Tag this event with tag bit 3 "
-			 "for retirement counting with execution_event.",
-		 .bit = 19,
-		},
 	 },
 	},
 
@@ -1358,26 +1298,6 @@ pentium4_event_t pentium4_events[] = {
 			 "double-precisions operands.",
 		 .bit = 15,
 		},
-		{.name = "TAG0",
-		 .desc = "Tag this event with tag bit 0 "
-			 "for retirement counting with execution_event.",
-		 .bit = 16,
-		},
-		{.name = "TAG1",
-		 .desc = "Tag this event with tag bit 1 "
-			 "for retirement counting with execution_event.",
-		 .bit = 17,
-		},
-		{.name = "TAG2",
-		 .desc = "Tag this event with tag bit 2 "
-			 "for retirement counting with execution_event.",
-		 .bit = 18,
-		},
-		{.name = "TAG3",
-		 .desc = "Tag this event with tag bit 3 "
-			 "for retirement counting with execution_event.",
-		 .bit = 19,
-		},
 	 },
 	},
 
@@ -1394,26 +1314,6 @@ pentium4_event_t pentium4_events[] = {
 			 "operands in memory or MMX registers.",
 		 .bit = 15,
 		},
-		{.name = "TAG0",
-		 .desc = "Tag this event with tag bit 0 "
-			 "for retirement counting with execution_event.",
-		 .bit = 16,
-		},
-		{.name = "TAG1",
-		 .desc = "Tag this event with tag bit 1 "
-			 "for retirement counting with execution_event.",
-		 .bit = 17,
-		},
-		{.name = "TAG2",
-		 .desc = "Tag this event with tag bit 2 "
-			 "for retirement counting with execution_event.",
-		 .bit = 18,
-		},
-		{.name = "TAG3",
-		 .desc = "Tag this event with tag bit 3 "
-			 "for retirement counting with execution_event.",
-		 .bit = 19,
-		},
 	 },
 	},
 
@@ -1430,26 +1330,6 @@ pentium4_event_t pentium4_events[] = {
 			 "operands in memory or MMX registers.",
 		 .bit = 15,
 		},
-		{.name = "TAG0",
-		 .desc = "Tag this event with tag bit 0 "
-			 "for retirement counting with execution_event.",
-		 .bit = 16,
-		},
-		{.name = "TAG1",
-		 .desc = "Tag this event with tag bit 1 "
-			 "for retirement counting with execution_event.",
-		 .bit = 17,
-		},
-		{.name = "TAG2",
-		 .desc = "Tag this event with tag bit 2 "
-			 "for retirement counting with execution_event.",
-		 .bit = 18,
-		},
-		{.name = "TAG3",
-		 .desc = "Tag this event with tag bit 3 "
-			 "for retirement counting with execution_event.",
-		 .bit = 19,
-		},
 	 },
 	},
 
@@ -1464,26 +1344,6 @@ pentium4_event_t pentium4_events[] = {
 		 .desc = "Count all x87 FP uops.",
 		 .bit = 15,
 		},
-		{.name = "TAG0",
-		 .desc = "Tag this event with tag bit 0 "
-			 "for retirement counting with execution_event.",
-		 .bit = 16,
-		},
-		{.name = "TAG1",
-		 .desc = "Tag this event with tag bit 1 "
-			 "for retirement counting with execution_event.",
-		 .bit = 17,
-		},
-		{.name = "TAG2",
-		 .desc = "Tag this event with tag bit 2 "
-			 "for retirement counting with execution_event.",
-		 .bit = 18,
-		},
-		{.name = "TAG3",
-		 .desc = "Tag this event with tag bit 3 "
-			 "for retirement counting with execution_event.",
-		 .bit = 19,
-		},
 	 },
 	},
 
