@@ -1,5 +1,6 @@
 /*
  * pfmlib_i386_pm.c : support for the P6 processor family (family=6)
+ * 		      incl. Pentium II, Pentium III, Pentium Pro, Pentium M
  *
  * Copyright (c) 2005-2006 Hewlett-Packard Development Company, L.P.
  * Contributed by Stephane Eranian <eranian@hpl.hp.com>
@@ -25,14 +26,14 @@
 #include <ctype.h>
 #include <string.h>
 #include <stdio.h>
-
+#include <stdlib.h>
 /* public headers */
 #include <perfmon/pfmlib_i386_p6.h>
 
 /* private headers */
 #include "pfmlib_priv.h"			/* library private */
 #include "pfmlib_i386_p6_priv.h"		/* architecture private */
-#include "i386_p6_events.h"			/* event tables */
+#include "i386_p6_events.h"			/* event tables P6 and Pentium M */
 
 /* let's define some handy shortcuts! */
 #define sel_event_mask	perfsel.sel_event_mask
@@ -50,92 +51,66 @@ static char * pfm_i386_p6_get_event_name(unsigned int i);
 static pme_i386_p6_entry_t *i386_pe;
 static unsigned int i386_p6_num_events;
 
-static inline unsigned int cpuid_eax(unsigned int op)
-{
-	unsigned int eax;
-
-	__asm__("pushl %%ebx; cpuid; popl %%ebx"
-			: "=a" (eax)
-			: "0" (op)
-			: "cx", "dx");
-	return eax;
-}
+#define PFMLIB_I386_P6_HAS_COMBO(_e) ((i386_pe[_e].pme_flags & PFMLIB_I386_P6_UMASK_COMBO) != 0)
 
+#define PFMLIB_I386_P6_ALL_FLAGS \
+	(PFM_I386_P6_SEL_INV|PFM_I386_P6_SEL_EDGE)
 /*
- * Verify we are on an Intel processor
- * XXX: siwtch to CPUID to avoid dependency on /proc/cpuinfo
+ * Description of the PMC register mappings use by
+ * this module.
+ * pfp_pmcs[].reg_num:
+ *	0 -> PMC0 -> PERFEVTSEL0 -> MSR @ 0x186
+ *	1 -> PMC1 -> PERFEVTSEL1 -> MSR @ 0x187
+ * pfp_pmds[].reg_num:
+ *	0 -> PMD0 -> PERFCTR0 -> MSR @ 0xc1
+ *	1 -> PMD1 -> PERFCTR1 -> MSR @ 0xc2
  */
+#define I386_P6_SEL_BASE	0x186
+#define I386_P6_CTR_BASE	0xc1
+
 static int
-check_intel_vendor(void)
+pfm_i386_detect_common(void)
 {
-	FILE *fp1;	
-	char buffer[128], *p, *value;
+	int ret, family;
+	char buffer[128];
 
-	fp1 = fopen("/proc/cpuinfo", "r");
-	if (fp1 == NULL) return 0;
+	ret = __pfm_getcpuinfo_attr("vendor_id", buffer, sizeof(buffer));
+	if (ret == -1)
+		return PFMLIB_ERR_NOTSUPP;
 
-	for (;;) {
-		buffer[0] = '\0';
+	if (strcmp(buffer, "GenuineIntel"))
+		return PFMLIB_ERR_NOTSUPP;
 
-		p  = fgets(buffer, 127, fp1);
-		if (p == NULL)
-			break;
+	ret = __pfm_getcpuinfo_attr("cpu family", buffer, sizeof(buffer));
+	if (ret == -1)
+		return PFMLIB_ERR_NOTSUPP;
 
-		/* skip  blank lines */
-		if (*p == '\n') continue;
+	family = atoi(buffer);
 
-		p = strchr(buffer, ':');
-		if (p == NULL)
-			break;
-
-		/*
-		 * p+2: +1 = space, +2= firt character
-		 * strlen()-1 gets rid of \n
-		 */
-		*p = '\0';
-		value = p+2;
-
-		value[strlen(value)-1] = '\0';
-
-		if (!strncmp("vendor_id", buffer, 9)) {
-			return strcmp(value, "GenuineIntel");
-		}
-	}
-	fclose(fp1);
-	return -1;
+	return family != 6 ? PFMLIB_ERR_NOTSUPP : PFMLIB_SUCCESS;
 }
 
 /*
- * detect only non Pentium P6 cores
+ * detect non Pentium M P6 cores
  */
 static int
 pfm_i386_p6_detect(void)
 {
-	unsigned long eax;
-	unsigned long family, model;
+	int ret, model;
+	char buffer[128];
 
-	/*
-	 * check that the core library supports enough registers
-	 */
-	if (PFMLIB_MAX_PMCS < PMU_I386_P6_NUM_PERFSEL) return PFMLIB_ERR_NOTSUPP;
-	if (PFMLIB_MAX_PMDS < PMU_I386_P6_NUM_PERFCTR) return PFMLIB_ERR_NOTSUPP;
+	ret = pfm_i386_detect_common();
+	if (ret != PFMLIB_SUCCESS)
+		return ret;
 
-	eax = cpuid_eax(1);
-
-	/*
-	 * we cannot check MSR_MISC_ENABLE from user level. Host
-	 * processoor may not have PMU.
-	 */
-	family = (eax>>8) & 0xf;
-	model  = (eax>>4) & 0xf;
-
-	if (family != 0x6)
+	ret = __pfm_getcpuinfo_attr("model", buffer, sizeof(buffer));
+	if (ret == -1)
 		return PFMLIB_ERR_NOTSUPP;
 
-	if (check_intel_vendor())
-		return PFMLIB_ERR_NOTSUPP;
+	model = atoi(buffer);
 
 	switch(model) {
+                case 3: /* Pentium II */
 		case 7: /* Pentium III Katmai */
 		case 8: /* Pentium III Coppermine */
 		case 9: /* Mobile Pentium III */
@@ -150,37 +125,33 @@ pfm_i386_p6_detect(void)
 	return PFMLIB_SUCCESS;
 }
 
+/*
+ * detect only Pentium M core
+ */
 static int
 pfm_i386_pm_detect(void)
 {
-	unsigned long eax;
-	unsigned long family, model;
+	int ret, model;
+	char buffer[128];
 
-	/*
-	 * check that the core library supports enough registers
-	 */
-	if (PFMLIB_MAX_PMCS < PMU_I386_P6_NUM_PERFSEL) return PFMLIB_ERR_NOTSUPP;
-	if (PFMLIB_MAX_PMDS < PMU_I386_P6_NUM_PERFCTR) return PFMLIB_ERR_NOTSUPP;
+	ret = pfm_i386_detect_common();
+	if (ret != PFMLIB_SUCCESS)
+		return ret;
 
-	eax = cpuid_eax(1);
+	ret = __pfm_getcpuinfo_attr("model", buffer, sizeof(buffer));
+	if (ret == -1)
+		return PFMLIB_ERR_NOTSUPP;
 
-	/*
-	 * we cannot check MSR_MISC_ENABLE from user level. Host
-	 * processor may not have PMU.
-	 */
-	family = (eax>>8) & 0xf;
-	model  = (eax>>4) & 0xf;
-
-	/*
-	 * only looking for Pentium M
-	 */
-	if (family != 0x6 || model != 13) return PFMLIB_ERR_NOTSUPP;
+	model = atoi(buffer);
+	if (model != 13)
+		return PFMLIB_ERR_NOTSUPP;
 
 	i386_pe = i386_pm_pe;
 	i386_p6_num_events = PME_I386_PM_EVENT_COUNT;
 
 	return PFMLIB_SUCCESS;
 }
+
 /*
  * Automatically dispatch events to corresponding counters following constraints.
  * Upon return the pfarg_regt structure is ready to be submitted to kernel
@@ -192,14 +163,15 @@ pfm_i386_p6_dispatch_counters(pfmlib_input_param_t *inp, pfmlib_i386_p6_input_pa
 	pfmlib_i386_p6_counter_t *cntrs;
 	pfm_i386_p6_sel_reg_t reg;
 	pfmlib_event_t *e;
-	pfmlib_reg_t *pc;
+	pfmlib_reg_t *pc, *pd;
 	pfmlib_regmask_t *r_pmcs;
 	unsigned long plm;
-	unsigned int i, j, cnt;
+	unsigned int i, j, cnt, k, umask;
 	unsigned int assign[PMU_I386_P6_NUM_COUNTERS];
 
 	e      = inp->pfp_events;
 	pc     = outp->pfp_pmcs;
+	pd     = outp->pfp_pmds;
 	cnt    = inp->pfp_event_count;
 	r_pmcs = &inp->pfp_unavail_pmcs;
 	cntrs  = param ? param->pfp_i386_p6_counters : NULL;
@@ -220,7 +192,19 @@ pfm_i386_p6_dispatch_counters(pfmlib_input_param_t *inp, pfmlib_i386_p6_input_pa
 			DPRINT(("event=%d invalid plm=%d\n", e[j].event, e[j].plm));
 			return PFMLIB_ERR_INVAL;
 		}
-		
+
+		if (e[j].flags & ~PFMLIB_I386_P6_ALL_FLAGS) {
+			DPRINT(("event=%d invalid flags=0x%lx\n", e[j].event, e[j].flags));
+			return PFMLIB_ERR_INVAL;
+		}
+
+		/*
+		 * check illegal unit masks combination
+		 */
+		if (e[j].num_masks > 1 && PFMLIB_I386_P6_HAS_COMBO(e[j].event) == 0) {
+			DPRINT(("event does not support unit mask combination\n"));
+			return PFMLIB_ERR_FEATCOMB;
+		}
 		/*
 		 * exclude restricted registers from assignement
 		 */
@@ -238,33 +222,42 @@ pfm_i386_p6_dispatch_counters(pfmlib_input_param_t *inp, pfmlib_i386_p6_input_pa
 		/* if plm is 0, then assume not specified per-event and use default */
 		plm = e[j].plm ? e[j].plm : inp->pfp_dfl_plm;
 
-		reg.sel_event_mask = i386_pe[e[j].event].pme_entry_code.pme_code.pme_emask;
-		reg.sel_unit_mask  = i386_pe[e[j].event].pme_entry_code.pme_code.pme_umask;
+		reg.sel_event_mask = i386_pe[e[j].event].pme_code;
+		/*
+		 * some events have only a single umask. We do not create
+		 * specific umask entry in this case. The umask code is taken
+		 * out of the (extended) event code (2nd byte)
+		 */
+		umask = (i386_pe[e[j].event].pme_code >> 8) & 0xff;
+
+		for(k=0; k < e[j].num_masks; k++) {
+			umask |= i386_pe[e[j].event].pme_umasks[e[j].unit_masks[k]].pme_ucode;
+		}
+		reg.sel_unit_mask  = umask;
 		reg.sel_usr        = plm & PFM_PLM3 ? 1 : 0;
 		reg.sel_os         = plm & PFM_PLM0 ? 1 : 0;
-		reg.sel_en         = 1; /* force enable bit to 1 */
 		reg.sel_int        = 1; /* force APIC int to 1 */
+		/*
+		 * only perfevtsel0 has an enable bit (allows atomic start/stop)
+		 */
+		if (assign[j] == 0) 
+			reg.sel_en = 1; /* force enable bit to 1 */
+
 		if (cntrs) {
-			reg.sel_cnt_mask   = cntrs[j].cnt_mask;
-
-			/*
-			 * certain events require edge to be set
-			 */
-			reg.sel_edge	   = cntrs[j].flags & PFM_I386_P6_SEL_EDGE
-					   ||  i386_pe[e[j].event].pme_entry_code.pme_code.pme_edge ? 1 : 0;
-
-			reg.sel_inv	   = cntrs[j].flags & PFM_I386_P6_SEL_INV ? 1 : 0;
+			reg.sel_cnt_mask = cntrs[j].cnt_mask;
+			reg.sel_edge	 = cntrs[j].flags & PFM_I386_P6_SEL_EDGE ? 1 : 0;
+			reg.sel_inv	 = cntrs[j].flags & PFM_I386_P6_SEL_INV ? 1 : 0;
 		}
 
-		/*
-		 * XXX: assumes perfmon2 mappings
-		 */
 		pc[j].reg_num     = assign[j];
-		pc[j].reg_pmd_num = assign[j];
-		pc[j].reg_evt_idx = j;
 		pc[j].reg_value   = reg.val;
+		pc[j].reg_addr    = I386_P6_SEL_BASE+assign[j];
 
-		__pfm_vbprintf("[perfsel%u=0x%lx emask=0x%lx umask=0x%lx os=%d usr=%d en=%d int=%d inv=%d edge=%d cnt_mask=%d] %s\n",
+		pd[j].reg_num  = assign[j];
+		pd[j].reg_addr = I386_P6_CTR_BASE+assign[j];
+
+		__pfm_vbprintf("[PERFEVTSEL%u(pmc%u)=0x%lx emask=0x%x umask=0x%x os=%d usr=%d en=%d int=%d inv=%d edge=%d cnt_mask=%d] %s\n",
+			assign[j],
 			assign[j],
 			reg.val,
 			reg.sel_event_mask,
@@ -277,9 +270,12 @@ pfm_i386_p6_dispatch_counters(pfmlib_input_param_t *inp, pfmlib_i386_p6_input_pa
 			reg.sel_edge,
 			reg.sel_cnt_mask,
 			i386_pe[e[j].event].pme_name);
+
+		__pfm_vbprintf("[PMC%u(pmd%u)]\n", pd[j].reg_num, pd[j].reg_num);
 	}
 	/* number of evtsel registers programmed */
 	outp->pfp_pmc_count = cnt;
+	outp->pfp_pmd_count = cnt;
 
 	return PFMLIB_SUCCESS;
 }
@@ -296,46 +292,17 @@ pfm_i386_p6_dispatch_events(pfmlib_input_param_t *inp, void *model_in, pfmlib_ou
 	return pfm_i386_p6_dispatch_counters(inp, mod_in, outp);
 }
 
-/*
- * Function used to print information about a specific events. More than
- * one event can be printed in case an event code is given rather than
- * a specific name. A callback function is used for printing.
- */
-static int
-pfm_i386_p6_print_info(unsigned int v, int (*pf)(const char *fmt,...))
-{
-	pme_i386_p6_entry_t *e;
-
-	if (v >= i386_p6_num_events || pf == NULL) return PFMLIB_ERR_INVAL;
-	e = i386_pe+v;
-
-	if (e->pme_desc) (*pf)("Desc   : %s\n", e->pme_desc);
-
-	return PFMLIB_SUCCESS;
-}
-
 static int
 pfm_i386_p6_get_event_code(unsigned int i, unsigned int cnt, int *code)
 {
 	if (cnt != PFMLIB_CNT_FIRST && cnt > 2)
 		return PFMLIB_ERR_INVAL;
-	
-	*code = i386_pe[i].pme_entry_code.pme_code.pme_emask;
+
+	*code = i386_pe[i].pme_code;
 
 	return PFMLIB_SUCCESS;
 }
 
-/*
- * This function is accessible directly to the user
- */
-int
-pfm_i386_p6_get_event_umask(unsigned int i, unsigned long *umask)
-{
-	if (i >= i386_p6_num_events || umask == NULL) return PFMLIB_ERR_INVAL;
-	*umask = 0;
-	return PFMLIB_SUCCESS;
-}
-	
 static void
 pfm_i386_p6_get_event_counters(unsigned int j, pfmlib_regmask_t *counters)
 {
@@ -408,48 +375,113 @@ pfm_i386_p6_get_event_description(unsigned int ev, char **str)
 	return PFMLIB_SUCCESS;
 }
 
+static char *
+pfm_i386_p6_get_event_mask_name(unsigned int ev, unsigned int midx)
+{
+	return i386_pe[ev].pme_umasks[midx].pme_uname;
+}
+
+static int
+pfm_i386_p6_get_event_mask_desc(unsigned int ev, unsigned int midx, char **str)
+{
+	char *s;
+
+	s = i386_pe[ev].pme_umasks[midx].pme_udesc;
+	if (s) {
+		*str = strdup(s);
+	} else {
+		*str = NULL;
+	}
+	return PFMLIB_SUCCESS;
+}
+
+static unsigned int
+pfm_i386_p6_get_num_event_masks(unsigned int ev)
+{
+	return i386_pe[ev].pme_numasks;
+}
+
+static int
+pfm_i386_p6_get_event_mask_code(unsigned int ev, unsigned int midx, unsigned int *code)
+{
+	*code = i386_pe[ev].pme_umasks[midx].pme_ucode;
+	return PFMLIB_SUCCESS;
+}
+	
+static int
+pfm_i386_p6_get_cycle_event(pfmlib_event_t *e)
+{
+	e->event = PME_I386_P6_CPU_CLK_UNHALTED;
+	return PFMLIB_SUCCESS;
+
+}
+
+static int
+pfm_i386_p6_get_inst_retired(pfmlib_event_t *e)
+{
+	e->event = PME_I386_P6_INST_RETIRED;
+	return PFMLIB_SUCCESS;
+}
+
+static int
+pfm_i386_pm_get_cycle_event(pfmlib_event_t *e)
+{
+	e->event = PME_I386_PM_CPU_CLK_UNHALTED;
+	return PFMLIB_SUCCESS;
+
+}
+
+static int
+pfm_i386_pm_get_inst_retired(pfmlib_event_t *e)
+{
+	e->event = PME_I386_PM_INST_RETIRED;
+	return PFMLIB_SUCCESS;
+}
+
 /* Generic P6 processor support (not incl. Pentium M) */
 pfm_pmu_support_t i386_p6_support={
-	.pmu_name		= "P6 Processor Family",
+	.pmu_name		= "Intel P6 Processor Family",
 	.pmu_type		= PFMLIB_I386_P6_PMU,
 	.pme_count		= PME_I386_P6_EVENT_COUNT,
 	.pmc_count		= PMU_I386_P6_NUM_PERFSEL,
 	.pmd_count		= PMU_I386_P6_NUM_PERFCTR,
 	.num_cnt		= PMU_I386_P6_NUM_COUNTERS,
-	.cycle_event		= PME_I386_P6_CPU_CLK_UNHALTED,
-	.inst_retired_event	= PME_I386_P6_INST_RETIRED,
 	.get_event_code		= pfm_i386_p6_get_event_code,
 	.get_event_name		= pfm_i386_p6_get_event_name,
 	.get_event_counters	= pfm_i386_p6_get_event_counters,
-	.print_info		= pfm_i386_p6_print_info,
 	.dispatch_events	= pfm_i386_p6_dispatch_events,
 	.pmu_detect		= pfm_i386_p6_detect,
 	.get_impl_pmcs		= pfm_i386_p6_get_impl_perfsel,
 	.get_impl_pmds		= pfm_i386_p6_get_impl_perfctr,
 	.get_impl_counters	= pfm_i386_p6_get_impl_counters,
 	.get_hw_counter_width	= pfm_i386_p6_get_hw_counter_width,
-	.get_event_desc         = pfm_i386_p6_get_event_description
+	.get_event_desc         = pfm_i386_p6_get_event_description,
+	.get_cycle_event	= pfm_i386_p6_get_cycle_event,
+	.get_inst_retired_event = pfm_i386_p6_get_inst_retired
 };
 
 /* Pentium M support */
 pfm_pmu_support_t i386_pm_support={
-	.pmu_name		= "Pentium M",
-	.pmu_type		= PFMLIB_I386_P6_PMU,
+	.pmu_name		= "Intel Pentium M",
+	.pmu_type		= PFMLIB_I386_PM_PMU,
 	.pme_count		= PME_I386_PM_EVENT_COUNT,
 	.pmc_count		= PMU_I386_P6_NUM_PERFSEL,
 	.pmd_count		= PMU_I386_P6_NUM_PERFCTR,
 	.num_cnt		= PMU_I386_P6_NUM_COUNTERS,
-	.cycle_event		= PME_I386_PM_CPU_CLK_UNHALTED,
-	.inst_retired_event	= PME_I386_PM_INST_RETIRED,
 	.get_event_code		= pfm_i386_p6_get_event_code,
 	.get_event_name		= pfm_i386_p6_get_event_name,
 	.get_event_counters	= pfm_i386_p6_get_event_counters,
-	.print_info		= pfm_i386_p6_print_info,
 	.dispatch_events	= pfm_i386_p6_dispatch_events,
 	.pmu_detect		= pfm_i386_pm_detect,
 	.get_impl_pmcs		= pfm_i386_p6_get_impl_perfsel,
 	.get_impl_pmds		= pfm_i386_p6_get_impl_perfctr,
 	.get_impl_counters	= pfm_i386_p6_get_impl_counters,
 	.get_hw_counter_width	= pfm_i386_p6_get_hw_counter_width,
-	.get_event_desc         = pfm_i386_p6_get_event_description
+	.get_event_desc         = pfm_i386_p6_get_event_description,
+	.get_num_event_masks	= pfm_i386_p6_get_num_event_masks,
+	.get_event_mask_name	= pfm_i386_p6_get_event_mask_name,
+	.get_event_mask_code	= pfm_i386_p6_get_event_mask_code,
+	.get_event_mask_desc	= pfm_i386_p6_get_event_mask_desc,
+	.get_cycle_event	= pfm_i386_pm_get_cycle_event,
+	.get_inst_retired_event = pfm_i386_pm_get_inst_retired
 };
