@@ -50,8 +50,8 @@
 #define evt_use_irange(e)	((e)->pfp_ita2_irange.rr_used)
 #define evt_use_drange(e)	((e)->pfp_ita2_drange.rr_used)
 
-#define evt_grp(e)	itanium2_pe[e].pme_qualifiers.pme_qual.pme_group
-#define evt_set(e)	itanium2_pe[e].pme_qualifiers.pme_qual.pme_set
+#define evt_grp(e)	(int)itanium2_pe[e].pme_qualifiers.pme_qual.pme_group
+#define evt_set(e)	(int)itanium2_pe[e].pme_qualifiers.pme_qual.pme_set
 #define evt_umask(e)	itanium2_pe[e].pme_umask
 
 
@@ -70,6 +70,28 @@
 
 static char * pfm_ita2_get_event_name(unsigned int i);
 
+/*
+ * Description of the PMC register mappings use by
+ * this module (as reported in pfmlib_reg_t.reg_num):
+ *
+ * 0 -> PMC0
+ * 1 -> PMC1
+ * n -> PMCn
+ *
+ * The following are in the model specific rr_br[]:
+ * IBR0 -> 0
+ * IBR1 -> 1
+ * ...
+ * IBR7 -> 7
+ * DBR0 -> 0
+ * DBR1 -> 1
+ * ...
+ * DBR7 -> 7
+ *
+ * We do not use a mapping table, instead we make up the
+ * values on the fly given the base.
+ */
+
 /*
  * The Itanium2 PMU has a bug in the fine mode implementation.
  * It only sees ranges with a granularity of two bundles.
@@ -83,12 +105,6 @@ pfm_ita2_detect(void)
 	int tmp;
 	int ret = PFMLIB_ERR_NOTSUPP;
 
-	/*
-	 * check that the core library supports enough registers
-	 */
-	if (PFMLIB_MAX_PMCS < PMU_ITA2_NUM_PMCS) return ret;
-	if (PFMLIB_MAX_PMDS < PMU_ITA2_NUM_PMDS) return ret;
-
 	tmp = pfm_ia64_get_cpu_family();
 	if (tmp == 0x1f) {
 		has_fine_mode_bug = 1;
@@ -289,7 +305,7 @@ valid_assign(pfmlib_event_t *e, unsigned int *as, pfmlib_regmask_t *r_pmcs, unsi
 	unsigned int i;
 	int c, failure;
 	int need_pmc5, need_pmc4;
-	int pmc5_evt = -1, pmc4_evt = -1;
+  	int pmc5_evt = -1, pmc4_evt = -1;
 
 	if (PFMLIB_DEBUG()) {
 		unsigned int j;
@@ -448,7 +464,7 @@ pfm_ita2_dispatch_counters(pfmlib_input_param_t *inp, pfmlib_ita2_input_param_t
 	pfmlib_ita2_input_param_t *param = mod_in;
 	pfm_ita2_pmc_reg_t reg;
 	pfmlib_event_t *e;
-	pfmlib_reg_t *pc;
+	pfmlib_reg_t *pc, *pd;
 	pfmlib_regmask_t *r_pmcs;
 	unsigned int i,j,k,l;
 	int ret;
@@ -458,6 +474,7 @@ pfm_ita2_dispatch_counters(pfmlib_input_param_t *inp, pfmlib_ita2_input_param_t
 
 	e      = inp->pfp_events;
 	pc     = outp->pfp_pmcs;
+	pd     = outp->pfp_pmds;
 	cnt    = inp->pfp_event_count;
 	r_pmcs = &inp->pfp_unavail_pmcs;
 
@@ -535,21 +552,27 @@ done:
 		 * This way we don't have to program something in PMC4 even when we don't use it
 		 */
 		pc[j].reg_num     = assign[j];
-		pc[j].reg_pmd_num = assign[j];
-		pc[j].reg_evt_idx = j;
 		pc[j].reg_value   = reg.pmc_val;
+		pc[j].reg_addr    = assign[j];
 
-		__pfm_vbprintf("[pmc%u=0x%06lx thres=%d es=0x%02x plm=%d umask=0x%x pm=%d ism=0x%x oi=%d] %s\n",
-					assign[j], reg.pmc_val,
-					reg.pmc_thres,
-					reg.pmc_es,reg.pmc_plm,
-					reg.pmc_umask, reg.pmc_pm,
-					reg.pmc_ism,
-					reg.pmc_oi,
-					itanium2_pe[e[j].event].pme_name);
+		pd[j].reg_num  = assign[j];
+		pd[j].reg_addr = assign[j];
+
+		__pfm_vbprintf("[PMC%u(pmc%u)=0x%06lx thres=%d es=0x%02x plm=%d umask=0x%x pm=%d ism=0x%x oi=%d] %s\n",
+				assign[j],
+				assign[j],
+				reg.pmc_val,
+				reg.pmc_thres,
+				reg.pmc_es,reg.pmc_plm,
+				reg.pmc_umask, reg.pmc_pm,
+				reg.pmc_ism,
+				reg.pmc_oi,
+				itanium2_pe[e[j].event].pme_name);
+		__pfm_vbprintf("[PMD%u(pmd%u)]\n", pd[j].reg_num, pd[j].reg_num);
 	}
 	/* number of PMC registers programmed */
 	outp->pfp_pmc_count = cnt;
+	outp->pfp_pmd_count = cnt;
 
 	return PFMLIB_SUCCESS;
 }
@@ -559,12 +582,17 @@ pfm_dispatch_iear(pfmlib_input_param_t *inp, pfmlib_ita2_input_param_t *mod_in,
 {
 	pfm_ita2_pmc_reg_t reg;
 	pfmlib_ita2_input_param_t *param = mod_in;
-	pfmlib_reg_t *pc = outp->pfp_pmcs;
+	pfmlib_reg_t *pc, *pd;
 	pfmlib_ita2_input_param_t fake_param;
-	int pos = outp->pfp_pmc_count;
+	unsigned int pos1, pos2;
 	unsigned int i, count;
 
+	pc = outp->pfp_pmcs;
+	pd = outp->pfp_pmds;
+	pos1 = outp->pfp_pmc_count;
+	pos2 = outp->pfp_pmd_count;
 	count = inp->pfp_event_count;
+
 	for (i=0; i < count; i++) {
 		if (is_iear(inp->pfp_events[i].event)) break;
 	}
@@ -618,29 +646,35 @@ pfm_dispatch_iear(pfmlib_input_param_t *inp, pfmlib_ita2_input_param_t *mod_in,
 	if (pfm_regmask_isset(&inp->pfp_unavail_pmcs, 10))
 		return PFMLIB_ERR_NOASSIGN;
 
-	pc[pos].reg_num     = 10; /* PMC10 is I-EAR config register */
-	pc[pos].reg_pmd_num = PFMLIB_REG_PMD_NONE;
-	pc[pos].reg_evt_idx = PFMLIB_EVENT_NONE;
-	pc[pos++].reg_value = reg.pmc_val;
+	pc[pos1].reg_num     = 10; /* PMC10 is I-EAR config register */
+	pc[pos1].reg_value   = reg.pmc_val;
+	pc[pos1].reg_addr  = 10;
+	pos1++;
+	pd[pos2].reg_num     = 0; 
+	pd[pos2++].reg_addr  = 0;
+	pd[pos2].reg_num     = 1; 
+	pd[pos2++].reg_addr  = 1;
 
 	if (param->pfp_ita2_iear.ear_mode == PFMLIB_ITA2_EAR_TLB_MODE) {
-		__pfm_vbprintf("[pmc10=0x%lx ctb=tlb plm=%d pm=%d ism=0x%x umask=0x%x]\n",
+		__pfm_vbprintf("[PMC10(pmc10)=0x%lx ctb=tlb plm=%d pm=%d ism=0x%x umask=0x%x]\n",
 			reg.pmc_val,
 			reg.pmc10_ita2_tlb_reg.iear_plm,
 			reg.pmc10_ita2_tlb_reg.iear_pm,
 			reg.pmc10_ita2_tlb_reg.iear_ism,
 			reg.pmc10_ita2_tlb_reg.iear_umask);
 	} else {
-		__pfm_vbprintf("[pmc10=0x%lx ctb=cache plm=%d pm=%d ism=0x%x umask=0x%x]\n",
+		__pfm_vbprintf("[PMC10(pmc10)=0x%lx ctb=cache plm=%d pm=%d ism=0x%x umask=0x%x]\n",
 			reg.pmc_val,
 			reg.pmc10_ita2_cache_reg.iear_plm,
 			reg.pmc10_ita2_cache_reg.iear_pm,
 			reg.pmc10_ita2_cache_reg.iear_ism,
 			reg.pmc10_ita2_cache_reg.iear_umask);
 	}
+	__pfm_vbprintf("[PMD0(pmd0)]\n[PMD1(pmd1)\n");
 
 	/* update final number of entries used */
-	outp->pfp_pmc_count = pos;
+	outp->pfp_pmc_count = pos1;
+	outp->pfp_pmd_count = pos2;
 
 	return PFMLIB_SUCCESS;
 }
@@ -650,12 +684,17 @@ pfm_dispatch_dear(pfmlib_input_param_t *inp, pfmlib_ita2_input_param_t *mod_in,
 {
 	pfm_ita2_pmc_reg_t reg;
 	pfmlib_ita2_input_param_t *param = mod_in;
-	pfmlib_reg_t *pc = outp->pfp_pmcs;
+	pfmlib_reg_t *pc, *pd;
 	pfmlib_ita2_input_param_t fake_param;
-	int pos = outp->pfp_pmc_count;
+	unsigned int pos1, pos2;
 	unsigned int i, count;
 
+	pc = outp->pfp_pmcs;
+	pd = outp->pfp_pmds;
+	pos1 = outp->pfp_pmc_count;
+	pos2 = outp->pfp_pmd_count;
 	count = inp->pfp_event_count;
+
 	for (i=0; i < count; i++) {
 		if (is_dear(inp->pfp_events[i].event)) break;
 	}
@@ -703,12 +742,18 @@ pfm_dispatch_dear(pfmlib_input_param_t *inp, pfmlib_ita2_input_param_t *mod_in,
 	if (pfm_regmask_isset(&inp->pfp_unavail_pmcs, 11))
 		return PFMLIB_ERR_NOASSIGN;
 
-	pc[pos].reg_num     = 11;  /* PMC11 is D-EAR config register */
-	pc[pos].reg_pmd_num = PFMLIB_REG_PMD_NONE;
-	pc[pos].reg_evt_idx = PFMLIB_EVENT_NONE;
-	pc[pos++].reg_value = reg.pmc_val;
+	pc[pos1].reg_num     = 11;  /* PMC11 is D-EAR config register */
+	pc[pos1].reg_value   = reg.pmc_val;
+	pc[pos1].reg_addr  = 11;
+	pos1++;
+	pd[pos2].reg_num     = 2; 
+	pd[pos2++].reg_addr  = 2;
+	pd[pos2].reg_num     = 3; 
+	pd[pos2++].reg_addr  = 3;
+	pd[pos2].reg_num     = 17; 
+	pd[pos2++].reg_addr  = 17;
 
-	__pfm_vbprintf("[pmc11=0x%lx mode=%s plm=%d pm=%d ism=0x%x umask=0x%x]\n",
+	__pfm_vbprintf("[PMC11(pmc11)=0x%lx mode=%s plm=%d pm=%d ism=0x%x umask=0x%x]\n",
 			reg.pmc_val,
 			reg.pmc11_ita2_reg.dear_mode == 0 ? "L1D" :
 			(reg.pmc11_ita2_reg.dear_mode == 1 ? "L1DTLB" : "ALAT"),
@@ -716,10 +761,12 @@ pfm_dispatch_dear(pfmlib_input_param_t *inp, pfmlib_ita2_input_param_t *mod_in,
 			reg.pmc11_ita2_reg.dear_pm,
 			reg.pmc11_ita2_reg.dear_ism,
 			reg.pmc11_ita2_reg.dear_umask);
+	__pfm_vbprintf("[PMD2(pmd2)]\n[PMD3(pmd3)\nPMD17(pmd17)\n");
 
 
 	/* update final number of entries used */
-	outp->pfp_pmc_count = pos;
+	outp->pfp_pmc_count = pos1;
+	outp->pfp_pmd_count = pos2;
 
 	return PFMLIB_SUCCESS;
 }
@@ -753,8 +800,6 @@ pfm_dispatch_opcm(pfmlib_input_param_t *inp, pfmlib_ita2_input_param_t *mod_in,
 			reg.pmc8_9_ita2_reg.opcm_inv   = 0;
 		}
 
-		memset(pc+pos, 0, sizeof(pfmlib_reg_t));
-
 		/* force bit 2 to 1 */
 		reg.pmc8_9_ita2_reg.opcm_bit2 = 1;
 
@@ -762,9 +807,8 @@ pfm_dispatch_opcm(pfmlib_input_param_t *inp, pfmlib_ita2_input_param_t *mod_in,
 			return PFMLIB_ERR_NOASSIGN;
 
 		pc[pos].reg_num     = 8;
-		pc[pos].reg_pmd_num = PFMLIB_REG_PMD_NONE;
-		pc[pos].reg_evt_idx = PFMLIB_EVENT_NONE;
-		pc[pos++].reg_value = reg.pmc_val;
+		pc[pos].reg_value   = reg.pmc_val;
+		pc[pos++].reg_addr  = 8;
 
 		/*
 		 * will be constrained by PMC8
@@ -780,7 +824,7 @@ pfm_dispatch_opcm(pfmlib_input_param_t *inp, pfmlib_ita2_input_param_t *mod_in,
 			if (has_2nd_pair || has_1st_pair == 0) pmc15.pmc15_ita2_reg.opcmc_ibrp2_pmc8 = 0;
 		}
 
-		__pfm_vbprintf("[pmc8=0x%lx m=%d i=%d f=%d b=%d match=0x%x mask=0x%x inv=%d ig_ad=%d]\n",
+		__pfm_vbprintf("[PMC8(pmc8)=0x%lx m=%d i=%d f=%d b=%d match=0x%x mask=0x%x inv=%d ig_ad=%d]\n",
 				reg.pmc_val,
 				reg.pmc8_9_ita2_reg.opcm_m,
 				reg.pmc8_9_ita2_reg.opcm_i,
@@ -798,8 +842,6 @@ pfm_dispatch_opcm(pfmlib_input_param_t *inp, pfmlib_ita2_input_param_t *mod_in,
 		 */
 		if (check_inst_retired_events(inp, NULL) != inp->pfp_event_count) return PFMLIB_ERR_FEATCOMB;
 
-		memset(pc+pos, 0, sizeof(pfmlib_reg_t));
-
 		reg.pmc_val = param->pfp_ita2_pmc9.pmc_val;
 
 		/* ig_ad, inv are ignored for PMC9, to avoid confusion we force default values */
@@ -812,8 +854,9 @@ pfm_dispatch_opcm(pfmlib_input_param_t *inp, pfmlib_ita2_input_param_t *mod_in,
 		if (pfm_regmask_isset(&inp->pfp_unavail_pmcs, 9))
 			return PFMLIB_ERR_NOASSIGN;
 
-		pc[pos].reg_num     = 9;
-		pc[pos++].reg_value = reg.pmc_val;
+		pc[pos].reg_num    = 9;
+		pc[pos].reg_value  = reg.pmc_val;
+		pc[pos++].reg_addr = 9;
 
 		/*
 		 * will be constrained by PMC9
@@ -828,7 +871,7 @@ pfm_dispatch_opcm(pfmlib_input_param_t *inp, pfmlib_ita2_input_param_t *mod_in,
 		if (has_1st_pair || has_2nd_pair == 0) pmc15.pmc15_ita2_reg.opcmc_ibrp1_pmc9 = 0;
 		if (has_2nd_pair || has_1st_pair == 0) pmc15.pmc15_ita2_reg.opcmc_ibrp3_pmc9 = 0;
 
-		__pfm_vbprintf("[pmc9=0x%lx m=%d i=%d f=%d b=%d match=0x%x mask=0x%x]\n",
+		__pfm_vbprintf("[PMC9(pmc9)=0x%lx m=%d i=%d f=%d b=%d match=0x%x mask=0x%x]\n",
 				reg.pmc_val,
 				reg.pmc8_9_ita2_reg.opcm_m,
 				reg.pmc8_9_ita2_reg.opcm_i,
@@ -838,15 +881,15 @@ pfm_dispatch_opcm(pfmlib_input_param_t *inp, pfmlib_ita2_input_param_t *mod_in,
 				reg.pmc8_9_ita2_reg.opcm_mask);
 
 	}
-	memset(pc+pos, 0, sizeof(pfmlib_reg_t));
 
 	if (pfm_regmask_isset(&inp->pfp_unavail_pmcs, 15))
 		return PFMLIB_ERR_NOASSIGN;
 
-	pc[pos].reg_num     = 15;
-	pc[pos++].reg_value = pmc15.pmc_val;
+	pc[pos].reg_num    = 15;
+	pc[pos].reg_value  = pmc15.pmc_val;
+	pc[pos++].reg_addr = 15;
 
-	__pfm_vbprintf("[pmc15=0x%lx ibrp0_pmc8=%d ibrp1_pmc9=%d ibrp2_pmc8=%d ibrp3_pmc9=%d]\n",
+	__pfm_vbprintf("[PMC15(pmc15)=0x%lx ibrp0_pmc8=%d ibrp1_pmc9=%d ibrp2_pmc8=%d ibrp3_pmc9=%d]\n",
 			pmc15.pmc_val,
 			pmc15.pmc15_ita2_reg.opcmc_ibrp0_pmc8,
 			pmc15.pmc15_ita2_reg.opcmc_ibrp1_pmc9,
@@ -865,13 +908,19 @@ pfm_dispatch_btb(pfmlib_input_param_t *inp, pfmlib_ita2_input_param_t *mod_in, p
 	pfmlib_event_t *e= inp->pfp_events;
 	pfm_ita2_pmc_reg_t reg;
 	pfmlib_ita2_input_param_t *param = mod_in;
-	pfmlib_reg_t *pc = outp->pfp_pmcs;
+	pfmlib_reg_t *pc, *pd;
 	pfmlib_ita2_input_param_t fake_param;
 	int found_btb = 0, found_bad_dear = 0;
 	int has_btb_param;
-	unsigned int i, pos = outp->pfp_pmc_count;
+	unsigned int i, pos1, pos2;
 	unsigned int count;
 
+	pc = outp->pfp_pmcs;
+	pd = outp->pfp_pmds;
+	pos1 = outp->pfp_pmc_count;
+	pos2 = outp->pfp_pmd_count;
+
+
 	/*
 	 * explicit BTB settings
 	 */
@@ -963,12 +1012,14 @@ assign_zero:
 	if (pfm_regmask_isset(&inp->pfp_unavail_pmcs, 12))
 		return PFMLIB_ERR_NOASSIGN;
 
-	pc[pos].reg_num     = 12;
-	pc[pos].reg_pmd_num = PFMLIB_REG_PMD_NONE;
-	pc[pos].reg_evt_idx = PFMLIB_EVENT_NONE;
-	pc[pos++].reg_value = reg.pmc_val;
+	memset(pc+pos1, 0, sizeof(pfmlib_reg_t));
 
-	__pfm_vbprintf("[pmc12=0x%lx plm=%d pm=%d ds=%d tm=%d ptm=%d ppm=%d brt=%d]\n",
+	pc[pos1].reg_num   = 12;
+	pc[pos1].reg_value = reg.pmc_val;
+	pc[pos1].reg_addr  = 12;
+	pos1++;
+
+	__pfm_vbprintf("[PMC12(pmc12)=0x%lx plm=%d pm=%d ds=%d tm=%d ptm=%d ppm=%d brt=%d]\n",
 				reg.pmc_val,
 				reg.pmc12_ita2_reg.btbc_plm,
 				reg.pmc12_ita2_reg.btbc_pm,
@@ -977,9 +1028,19 @@ assign_zero:
 				reg.pmc12_ita2_reg.btbc_ptm,
 				reg.pmc12_ita2_reg.btbc_ppm,
 				reg.pmc12_ita2_reg.btbc_brt);
+	/*
+	 * PMD16 is included in list of used PMD
+	 */
+	for(i=8; i < 17; i++, pos2++) {
+		pd[pos2].reg_num = i;
+		pd[pos2].reg_addr = i;
+		__pfm_vbprintf("[PMD%u(pmd%u)]\n", pd[pos2].reg_num, pd[pos2].reg_num);
+	}
+
 
 	/* update final number of entries used */
-	outp->pfp_pmc_count = pos;
+	outp->pfp_pmc_count = pos1;
+	outp->pfp_pmd_count = pos2;
 
 	return PFMLIB_SUCCESS;
 }
@@ -1072,9 +1133,11 @@ do_normal_rr(unsigned long start, unsigned long end,
 
 	br[*idx].reg_num     = *reg_idx;
 	br[*idx].reg_value   = l_addr;
+	br[*idx].reg_addr    = *reg_idx;
 
 	br[*idx+1].reg_num   = *reg_idx+1;
 	br[*idx+1].reg_value = db.val;
+	br[*idx+1].reg_addr  = *reg_idx+1;
 
 	*idx     += 2;
 	*reg_idx += 2;
@@ -1183,8 +1246,10 @@ compute_fine_rr(pfmlib_ita2_input_rr_t *irr, int dfl_plm, int n, int *base_idx,
 
 		br[0].reg_num   = reg_idx;
 		br[0].reg_value = addr;
+		br[0].reg_addr  = reg_idx;
 		br[1].reg_num   = reg_idx+1;
 		br[1].reg_value = db.val;
+		br[1].reg_addr  = reg_idx+1;
 
 		/*
 		 * setup upper limit pair
@@ -1207,9 +1272,11 @@ compute_fine_rr(pfmlib_ita2_input_rr_t *irr, int dfl_plm, int n, int *base_idx,
 
 		br[2].reg_num   = reg_idx+4;
 		br[2].reg_value = addr;
+		br[2].reg_addr  = reg_idx+4;
 
 		br[3].reg_num   = reg_idx+5;
 		br[3].reg_value = db.val;
+		br[3].reg_addr  = reg_idx+5;
 
 		if (PFMLIB_VERBOSE()) print_one_range(in_rr, out_rr, br, 0, 2, 1, irr->rr_flags);
 	}
@@ -1273,6 +1340,7 @@ found:
 	/* when the event is not IA64_INST_RETIRED, then we MUST use ibrp0 */
 	br[0].reg_num   = reg_idx;
 	br[0].reg_value = p_start;
+	br[0].reg_addr  = reg_idx;
 
 	db.val        = 0;
 	db.db.db_mask = ~((1UL << m)-1);
@@ -1281,6 +1349,7 @@ found:
 
 	br[1].reg_num   = reg_idx + 1;
 	br[1].reg_value = db.val;
+	br[1].reg_addr  = reg_idx + 1;
 
 	out_rr->rr_soff = start - p_start;
 	out_rr->rr_eoff = p_end - end;
@@ -1326,7 +1395,7 @@ compute_normal_rr(pfmlib_ita2_input_rr_t *irr, int dfl_plm, int n, int *base_idx
 				4 - (reg_idx>>1), /* how many pairs available */
 				0,
 				&br_index,
-				&reg_idx, in_rr->rr_plm ? in_rr->rr_plm : (unsigned long)dfl_plm);
+				&reg_idx, in_rr->rr_plm ? in_rr->rr_plm : dfl_plm);
 
 		DPRINT(("br_index=%d reg_idx=%d\n", br_index, reg_idx));
 
@@ -1510,11 +1579,10 @@ pfm_dispatch_irange(pfmlib_input_param_t *inp, pfmlib_ita2_input_param_t *mod_in
 		return PFMLIB_ERR_NOASSIGN;
 
 	pc[pos].reg_num     = 14;
-	pc[pos].reg_pmd_num = PFMLIB_REG_PMD_NONE;
-	pc[pos].reg_evt_idx = PFMLIB_EVENT_NONE;
-	pc[pos++].reg_value = reg.pmc_val;
+	pc[pos].reg_value   = reg.pmc_val;
+	pc[pos++].reg_addr  = 14;
 
-	__pfm_vbprintf("[pmc14=0x%lx ibrp0=%d ibrp1=%d ibrp2=%d ibrp3=%d fine=%d]\n",
+	__pfm_vbprintf("[PMC14(pmc14)=0x%lx ibrp0=%d ibrp1=%d ibrp2=%d ibrp3=%d fine=%d]\n",
 			reg.pmc_val,
 			reg.pmc14_ita2_reg.iarc_ibrp0,
 			reg.pmc14_ita2_reg.iarc_ibrp1,
@@ -1679,11 +1747,10 @@ pfm_dispatch_drange(pfmlib_input_param_t *inp, pfmlib_ita2_input_param_t *mod_in
 		return PFMLIB_ERR_NOASSIGN;
 
 	pc[pos].reg_num     = 13;
-	pc[pos].reg_pmd_num = PFMLIB_REG_PMD_NONE;
-	pc[pos].reg_evt_idx = PFMLIB_EVENT_NONE;
-	pc[pos++].reg_value = pmc13.pmc_val;
+	pc[pos].reg_value   = pmc13.pmc_val;
+	pc[pos++].reg_addr  = 13;
 
-	__pfm_vbprintf("[pmc13=0x%lx cfg_dbrp0=%d cfg_dbrp1=%d cfg_dbrp2=%d cfg_dbrp3=%d ena_dbrp0=%d ena_dbrp1=%d ena_dbrp2=%d ena_dbrp3=%d]\n",
+	__pfm_vbprintf("[PMC13(pmc13)=0x%lx cfg_dbrp0=%d cfg_dbrp1=%d cfg_dbrp2=%d cfg_dbrp3=%d ena_dbrp0=%d ena_dbrp1=%d ena_dbrp2=%d ena_dbrp3=%d]\n",
 			pmc13.pmc_val,
 			pmc13.pmc13_ita2_reg.darc_cfg_dbrp0,
 			pmc13.pmc13_ita2_reg.darc_cfg_dbrp1,
@@ -1897,82 +1964,13 @@ done:
 	return PFMLIB_SUCCESS;
 }
 
-/*
- * Function used to print information about a specific events. More than
- * one event can be printed in case an event code is given rather than
- * a specific name. A callback function is used for printing.
- */
-static int
-pfm_ita2_print_info(unsigned int v, int (*pf)(const char *fmt,...))
-{
-	pme_ita2_entry_t *e;
-        static const char *quals[]={ "[Instruction Address Range]", "[OpCode Match]", "[Data Address Range]" };
-	static const char *groups[]= {"???", "L1D Cache", "L2 Cache"};
-	long c;
-        int i;
-
-	if (v >= PME_ITA2_EVENT_COUNT || pf == NULL) return PFMLIB_ERR_INVAL;
-	e = itanium2_pe+v;
-
-	if (is_iear(e-itanium2_pe)) {
-		i= is_ear_tlb(e-itanium2_pe) ? 6 : 7;
-	} else {
-		i=3;
-	}
-	(*pf)("Umask  : ");
-	c = e->pme_umask;
-	for (; i >=0; i--) {
-		(*pf)("%c", c & 1<<i ? '1' : '0');
-	}
-	(*pf)("\n");
-
-	(*pf)( "EAR    : %s (%s)\n",
-		event_is_ear(e) ? (event_is_dear(e) ? "Data" : "Inst") : "No",
-		event_is_ear(e) ? (event_is_ear_tlb(e) ? "TLB Mode": (event_is_ear_alat(e) ? "ALAT Mode": "Cache Mode")): "N/A");
-	
-	(*pf)("BTB    : %s\n", event_is_btb(e) ? "Yes" : "No");
-	if (e->pme_maxincr > 1)
-		(*pf)("MaxIncr: %u  (Threshold [0-%u])\n", e->pme_maxincr,  e->pme_maxincr-1);
- 	else
-		(*pf)("MaxIncr: %u  (Threshold 0)\n", e->pme_maxincr);
-
-	(*pf)("Qual   : ");
-
-	c = e->pme_qualifiers.qual;
-	if ((c&0x7) == 0) {
-		(*pf)("None");
-	} else {
-		for (i=0; i < 3; i++ ) {
-                	if (c & 0x1) (*pf)("%s ", quals[i]);
-                	c >>= 1;
-        	}
-	}
-	(*pf)("\n");
-
-	if (e->pme_qualifiers.pme_qual.pme_group == PFMLIB_ITA2_EVT_NO_GRP)
-		(*pf)("Group  : None\n");
-	else {
-		unsigned long g = e->pme_qualifiers.pme_qual.pme_group;
-		const char *str =  g < 3 ? groups[g]: "none";
-		(*pf)("Group  : %s\n", str);
-	}
-	if (e->pme_qualifiers.pme_qual.pme_set == 0xf)
-		(*pf)("Set    : None\n");
-	else
-		(*pf)("Set    : %ld\n", e->pme_qualifiers.pme_qual.pme_set);
-
-	if (e->pme_desc) (*pf)("Desc   : %s\n", e->pme_desc);
-
-	return PFMLIB_SUCCESS;
-}
-
 static int
 pfm_ita2_get_event_code(unsigned int i, unsigned int cnt, int *code)
 {
 	if (cnt != PFMLIB_CNT_FIRST && (cnt < 4 || cnt > 7))
 		return PFMLIB_ERR_INVAL;
 
-	*code = itanium2_pe[i].pme_code;
+	*code = (int)itanium2_pe[i].pme_code;
 
 	return PFMLIB_SUCCESS;
 }
@@ -2000,7 +1998,7 @@ int
 pfm_ita2_get_event_set(unsigned int i, int *set)
 {
 	if (i >= PME_ITA2_EVENT_COUNT || set == NULL) return PFMLIB_ERR_INVAL;
-	*set = evt_set(i) == 0xf ? -1 : evt_set(i);
+	*set = evt_set(i) == 0xf ? PFMLIB_ITA2_EVT_NO_SET : evt_set(i);
 	return PFMLIB_SUCCESS;
 }
 
@@ -2107,6 +2105,21 @@ pfm_ita2_get_event_description(unsigned int ev, char **str)
 	return PFMLIB_SUCCESS;
 }
 
+static int
+pfm_ita2_get_cycle_event(pfmlib_event_t *e)
+{
+	e->event = PME_ITA2_CPU_CYCLES;
+	return PFMLIB_SUCCESS;
+
+}
+
+static int
+pfm_ita2_get_inst_retired(pfmlib_event_t *e)
+{
+	e->event = PME_ITA2_IA64_INST_RETIRED;
+	return PFMLIB_SUCCESS;
+}
+
 pfm_pmu_support_t itanium2_support={
 	.pmu_name		= "itanium2",
 	.pmu_type		= PFMLIB_ITANIUM2_PMU,
@@ -2114,17 +2127,16 @@ pfm_pmu_support_t itanium2_support={
 	.pmc_count		= PMU_ITA2_NUM_PMCS,
 	.pmd_count		= PMU_ITA2_NUM_PMDS,
 	.num_cnt		= PMU_ITA2_NUM_COUNTERS,
-	.cycle_event		= PME_ITA2_CPU_CYCLES,
-	.inst_retired_event	= PME_ITA2_IA64_INST_RETIRED,
 	.get_event_code		= pfm_ita2_get_event_code,
 	.get_event_name		= pfm_ita2_get_event_name,
 	.get_event_counters	= pfm_ita2_get_event_counters,
-	.print_info		= pfm_ita2_print_info,
 	.dispatch_events	= pfm_ita2_dispatch_events,
 	.pmu_detect		= pfm_ita2_detect,
 	.get_impl_pmcs		= pfm_ita2_get_impl_pmcs,
 	.get_impl_pmds		= pfm_ita2_get_impl_pmds,
 	.get_impl_counters	= pfm_ita2_get_impl_counters,
 	.get_hw_counter_width	= pfm_ita2_get_hw_counter_width,
-	.get_event_desc         = pfm_ita2_get_event_description
+	.get_event_desc         = pfm_ita2_get_event_description,
+	.get_cycle_event	= pfm_ita2_get_cycle_event,
+	.get_inst_retired_event = pfm_ita2_get_inst_retired
 };
