@@ -270,7 +270,6 @@ static int pentium4_dispatch_events(pfmlib_input_param_t *input,
 {
 	unsigned int assigned_pmcs[PENTIUM4_NUM_PMCS] = {0};
 	unsigned int event, event_mask, mask;
-	unsigned int tag_value, tag_enable;
 	unsigned int plm;
 	unsigned int i, j, k, m, n;
 	int escr, escr_pmc;
@@ -357,19 +356,12 @@ static int pentium4_dispatch_events(pfmlib_input_param_t *input,
 				 * specified by the caller are ignored.
 				 */
 				event_mask = 0;
-				tag_value = 0;
-				tag_enable = 0;
 				for (n = 0; n < input->pfp_events[i].num_masks; n++) {
 					mask = input->pfp_events[i].unit_masks[n];
 					if (mask < EVENT_MASK_BITS &&
 					    pentium4_events[event].event_masks[mask].name) {
 						event_mask |= (1 << pentium4_events[event].event_masks[mask].bit);
 					}
-					if (mask >= EVENT_MASK_BITS &&
-					    pentium4_events[event].event_masks[mask].name) {
-						tag_value |= (1 << (pentium4_events[event].event_masks[mask].bit - EVENT_MASK_BITS));
-						tag_enable = 1;
-					}
 				}
 
 				/* Set up the ESCR and CCCR register values. */
@@ -379,8 +371,8 @@ static int pentium4_dispatch_events(pfmlib_input_param_t *input,
 				escr_value.bits.t1_os        = 0; /* controlled by kernel */
 				escr_value.bits.t0_usr       = (plm & PFM_PLM3) ? 1 : 0;
 				escr_value.bits.t0_os        = (plm & PFM_PLM0) ? 1 : 0;
-				escr_value.bits.tag_enable   = tag_enable;
-				escr_value.bits.tag_value    = tag_value;
+				escr_value.bits.tag_enable   = 0; /* FIXME: What do we do with the "tag" entries? */
+				escr_value.bits.tag_value    = 0; /* FIXME: What do we do with the "tag" entries? */
 				escr_value.bits.event_mask   = event_mask;
 				escr_value.bits.event_select = pentium4_events[event].event_select;
 				escr_value.bits.reserved     = 0;
@@ -490,13 +482,6 @@ static int pentium4_pmu_detect(void)
 	 * we use model to detect model 2 which has one more counter IQ_ESCR1
 	 */
 	p4_model = atoi(buffer);
-	if (family != 15)
-		return PFMLIB_ERR_NOTSUPP;
-	/*
-	 * IQ_ESCR0, IQ_ESCR1 only for model 1 and 2 
-	 */
-	if (p4_model >2)
-		pentium4_support.pmc_count -= 2;
 
 	return family == 15 ? PFMLIB_SUCCESS : PFMLIB_ERR_NOTSUPP;
 }
@@ -513,16 +498,11 @@ static void pentium4_get_impl_pmcs(pfmlib_regmask_t *impl_pmcs)
 {
 	unsigned int i;
 
+	memset(impl_pmcs, 0, sizeof(*impl_pmcs));
+
 	for(i = 0; i < PENTIUM4_NUM_PMCS; i++) {
 		pfm_regmask_set(impl_pmcs, i);
 	}
-	/*
-	 * IQ_ESCR0, IQ_ESCR1 only available on model 1 and 2
-	 */
-	if (p4_model > 2) {
-		pfm_regmask_clr(impl_pmcs, 16);
-		pfm_regmask_clr(impl_pmcs, 48);
-	}
 }
 
 /**
@@ -537,6 +517,8 @@ static void pentium4_get_impl_pmds(pfmlib_regmask_t *impl_pmds)
 {
 	unsigned int i;
 
+	memset(impl_pmds, 0, sizeof(*impl_pmds));
+
 	for(i = 0; i < PENTIUM4_NUM_PMDS; i++) {
 		pfm_regmask_set(impl_pmds, i);
 	}
